﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ZylSoft.Gps;
using Color = System.Windows.Media.Color;

namespace WpfDemo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private GpsReceiver gpsReceiver = new GpsReceiver();
        private bool _cancelDetect;

        public MainWindow()
        {
            InitializeComponent();
            gpsReceiver.Connected += GpsReceiverOnConnected;
            gpsReceiver.Disconnected += GpsReceiverOnDisconnected;
            gpsReceiver.Received += GpsReceiverOnReceived;
            gpsReceiver.Detecting += GpsReceiverOnDetecting;
            gpsReceiver.LatitudeChanged += GpsReceiverOnLatitudeChanged;
            gpsReceiver.LongitudeChanged += GpsReceiverOnLongitudeChanged;
            gpsReceiver.AltitudeChanged += GpsReceiverOnAltitudeChanged;
            gpsReceiver.SpeedChanged += GpsReceiverOnSpeedChanged;
            gpsReceiver.CourseChanged += GpsReceiverOnCourseChanged;
            gpsReceiver.HeadingChanged += GpsReceiverOnHeadingChanged;
            gpsReceiver.SatellitesReceived += GpsReceiverSatellitesReceived;
            gpsReceiver.Position2DChanged += GpsReceiver_Position2DChanged;
        }

        private void GpsReceiver_Position2DChanged(object sender, DmsPositionEventArgs e)
        {
            this.Dispatcher.Invoke(new Action(delegate ()
            {
                gpsReceiver.DrawSatellitesToWpfCanvas(satellitesCanvas, (int)Math.Min(satellitesCanvas.Width, satellitesCanvas.Height) / 2, Colors.Black, Colors.White);
                gpsReceiver.DrawSatellitesSignalToWpfCanvas(satellitesLevelCanvas, (int)satellitesLevelCanvas.Width, (int)satellitesLevelCanvas.Height, Colors.Black);               
            }), null);
        }

        private void GpsReceiverSatellitesReceived(object sender, EventArgs e)
        {
            this.Dispatcher.Invoke(new Action(delegate () 
            { 
                gpsReceiver.DrawSatellitesToWpfCanvas(satellitesCanvas, (int)Math.Min(satellitesCanvas.Width, satellitesCanvas.Height) / 2, Colors.Black, Colors.White);
                gpsReceiver.DrawSatellitesSignalToWpfCanvas(satellitesLevelCanvas, (int)satellitesLevelCanvas.Width, (int)satellitesLevelCanvas.Height, Colors.Black);
            }), null);            
        }

        private void GpsReceiverOnDisconnected(object sender, ConnectionEventArgs connectionEventArgs)
        {            
            textBoxOut.Dispatcher.Invoke(new Action(delegate() { textBoxOut.Text += "Disconnected from: " + connectionEventArgs.Port + Environment.NewLine; textBoxOut.ScrollToEnd(); }), null);            
        }

        private void GpsReceiverOnConnected(object sender, ConnectionEventArgs connectionEventArgs)
        {
            textBoxOut.Dispatcher.Invoke(new Action(delegate() { textBoxOut.Text += "Connected to: " + connectionEventArgs.Port + Environment.NewLine; textBoxOut.ScrollToEnd(); }), null);           
        }

        private void GpsReceiverOnCourseChanged(object sender, ParamEventArgs paramEventArgs)
        {
            labelCourse.Dispatcher.Invoke(new Action(delegate() { labelCourse.Content = "Course: " + paramEventArgs.Value.ToString() + "º"; }), null);
        }

        private void GpsReceiverOnHeadingChanged(object sender, ParamEventArgs paramEventArgs)
        {
            this.Dispatcher.Invoke(new Action(delegate () 
            { 
                labelHeading.Content = "Heading: " + paramEventArgs.Value.ToString() + "º";
                gpsReceiver.DrawCompassWithSkinToWpfCanvas(compassCanvas, (int)(Math.Min(compassCanvas.Width, compassCanvas.Height) / 2));
            }), null);       
        }

        private void GpsReceiverOnSpeedChanged(object sender, ParamEventArgs paramEventArgs)
        {
            labelSpeed.Dispatcher.Invoke(new Action(delegate() { labelSpeed.Content = "Speed: " + paramEventArgs.Value.ToString() + " km/h"; }), null);
        }

        private void GpsReceiverOnAltitudeChanged(object sender, ParamEventArgs paramEventArgs)
        {
            labelAltitude.Dispatcher.Invoke(new Action(delegate() { labelAltitude.Content = "Altitude: " + paramEventArgs.Value.ToString() + " m"; }), null);
        }

        private void GpsReceiverOnLongitudeChanged(object sender, PositionEventArgs positionEventArgs)
        {
            string longitude = "Longitude: " + positionEventArgs.Degree + "º " + positionEventArgs.Minute + "' " +
                  positionEventArgs.Second + "'' " + positionEventArgs.Direction;
            labelLongitude.Dispatcher.Invoke(new Action(delegate() { labelLongitude.Content = longitude; }), null);
        }

        private void GpsReceiverOnLatitudeChanged(object sender, PositionEventArgs positionEventArgs)
        {
            string latitude = "Latitude: " + positionEventArgs.Degree + "º " + positionEventArgs.Minute + "' " +
                              positionEventArgs.Second + "'' " + positionEventArgs.Direction;
            labelLatitude.Dispatcher.Invoke(new Action(delegate() { labelLatitude.Content = latitude; }), null);
        }

        private void GpsReceiverOnDetecting(object sender, DetectEventArgs detectEventArgs)
        {            
            lblDetect.Dispatcher.Invoke(new Action(delegate() { lblDetect.Content = "Detecting: " + detectEventArgs.Port.ToString() + " " + detectEventArgs.BaudRate.ToString(); }), null);
            detectEventArgs.Cancel = _cancelDetect;
        }

        private void GpsReceiverOnReceived(object sender, DataEventArgs dataEventArgs)
        {
            this.Dispatcher.Invoke(new Action(delegate()
            {
                textBoxOut.Text += dataEventArgs.Buffer;
                textBoxOut.ScrollToEnd();

                labelUtcTime.Content = "UTC Time: " + gpsReceiver.Position.GetUtcDateTime().ToString();
                labelLocalTime.Content = "Local Time: " + gpsReceiver.Position.GetLocalDateTime().ToString();
            }), null);
        }

        private void buttonOpen_Click(object sender, RoutedEventArgs e)
        {
            if (listBoxPorts.SelectedItems.Count < 1 || listBoxBaudRates.SelectedItems.Count < 1)
            {
                MessageBox.Show("Select port and baud rate!");
                return;
            }
            string port = listBoxPorts.SelectedItems[0].ToString();
            gpsReceiver.Port = GpsReceiver.StringToSerialCommPort(port);

            if (gpsReceiver.Port == SerialCommPort.Custom)
            {
                gpsReceiver.CustomPortName = port;
            }  

            ListBoxItem item = (ListBoxItem)listBoxBaudRates.SelectedItems[0];
            int baudRate = Convert.ToInt32(item.Content.ToString());
            gpsReceiver.BaudRate = (SerialBaudRate)baudRate;
            if (!gpsReceiver.Open())
                MessageBox.Show("Cannot connect to " + gpsReceiver.Port.ToString());
        }

        private void buttonClose_Click(object sender, RoutedEventArgs e)
        {
            gpsReceiver.Close();
        }

        private void buttonDetect_Click(object sender, RoutedEventArgs e)
        {
            SerialCommPort port;
            SerialBaudRate baudRate;
            _cancelDetect = false;
            if (gpsReceiver.DetectGps(out port, out baudRate))
            {
                MessageBox.Show("Port: " + SerialCommPortExtensions.ToString(port) + ", Baud Rate: " + baudRate.ToString());

                listBoxPorts.Dispatcher.Invoke(new Action(delegate()
                {                    
                    for (int i = 0; i < listBoxPorts.Items.Count; i++)
                    {
                        string item = (string)listBoxPorts.Items[i];
                        if (item == SerialCommPortExtensions.ToString(port))
                        {
                            listBoxPorts.SelectedIndex = i;
                            //listBoxPorts.SelectedItem = item;                            
                            break;
                        }
                    }
                }));

                listBoxBaudRates.Dispatcher.Invoke(new Action(delegate()
                {
                    int i = 0;
                    foreach (ListBoxItem item in listBoxBaudRates.Items)
                    {                                       
                        if (Convert.ToInt32(item.Content.ToString()) == (int)baudRate)
                        {
                            listBoxBaudRates.SelectedIndex = i;                            
                            break;
                        }
                        i++;
                    }
                }));  
               
            }
            else
                MessageBox.Show("No GPS detected.");
            lblDetect.Content = string.Empty;
        }

        private void buttonCancel_Click(object sender, RoutedEventArgs e)
        {
            _cancelDetect = true;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            listBoxPorts.Items.Clear();
            String[] ports = ZylSoft.Gps.GpsReceiver.GetExistingCommPortNames();
            foreach (String i in ports)
            {
                listBoxPorts.Items.Add(i);
            }            
        }
    }
}
